@@ -114,19 +114,32 @@
 //!
 
 use super::RustBuffer;
+use std::os::raw::c_int;
 use std::sync::atomic::{AtomicUsize, Ordering};
 
 /// ForeignCallback is the Rust representation of a foreign language function.
 /// It is the basis for all callbacks interfaces. It is registered exactly once per callback interface,
 /// at library start up time.
 /// Calling this method is only done by generated objects which mirror callback interfaces objects in the foreign language.
-/// The `handle` is the key into a handle map on the other side of the FFI used to look up the foreign language object
-/// that implements the callback interface/trait.
-/// The `method` selector specifies the method that will be called on the object, by looking it up in a list of methods from
-/// the IDL. The index is 1 indexed. Note that the list of methods is generated by at uniffi from the IDL and used in all
-/// bindings: so we can rely on the method list being stable within the same run of uniffi.
-pub type ForeignCallback =
-    unsafe extern "C" fn(handle: u64, method: u32, args: RustBuffer) -> RustBuffer;
+///
+/// * The `handle` is the key into a handle map on the other side of the FFI used to look up the foreign language object
+///   that implements the callback interface/trait.
+/// * The `method` selector specifies the method that will be called on the object, by looking it up in a list of methods from
+///   the IDL. The index is 1 indexed. Note that the list of methods is generated by at uniffi from the IDL and used in all
+///   bindings: so we can rely on the method list being stable within the same run of uniffi.
+/// * `args` is a serialized buffer of arguments to the function. UniFFI will deserialize it before
+///   passing individual arguments to the user's callback.
+/// * `buf_ptr` is a pointer to where the resulting buffer will be written. UniFFI will allocate a
+///   buffer to write the result into.
+/// * A callback returns `-1` if an unexpected error happens, `0` if the call succeeded without a
+///   return value or `1` if data was written into the output buffer.
+///   Note: The output buffer might still contain 0 bytes of data.
+pub type ForeignCallback = unsafe extern "C" fn(
+    handle: u64,
+    method: u32,
+    args: RustBuffer,
+    buf_ptr: *mut RustBuffer,
+) -> c_int;
 
 /// The method index used by the Drop trait to communicate to the foreign language side that Rust has finished with it,
 /// and it can be deleted from the handle map.
